From c43f3dd58bf82449a10d6000918e3dd223fbf486 Mon Sep 17 00:00:00 2001
From: Michael Biebl <biebl@debian.org>
Date: Sat, 9 Feb 2013 09:09:47 +0100
Subject: [PATCH] Don't keep file descriptors open when forking

http://0pointer.de/public/systemd-man/daemon.html
http://code.activestate.com/recipes/278731-creating-a-daemon-the-python-way/
---
 src/firewalld |   23 ++++++++++++++++++++++-
 1 file changed, 22 insertions(+), 1 deletion(-)

diff --git a/src/firewalld b/src/firewalld
index a24f2eb..70fff7a 100755
--- a/src/firewalld
+++ b/src/firewalld
@@ -106,7 +106,28 @@ try:
         os.chdir("/")
         os.setsid()
         os.umask(os.umask(0077) | 0022)
-    
+
+        import resource		# Resource usage information.
+        maxfd = resource.getrlimit(resource.RLIMIT_NOFILE)[1]
+        if (maxfd == resource.RLIM_INFINITY):
+            maxfd = 1024
+
+        # Iterate through and close all file descriptors.
+        for fd in range(0, maxfd):
+            try:
+                os.close(fd)
+            except OSError:	# ERROR, fd wasn't open to begin with (ignored)
+                pass
+
+        # Redirect the standard I/O file descriptors to /dev/null
+        if (hasattr(os, "devnull")):
+            REDIRECT_TO = os.devnull
+        else:
+            REDIRECT_TO = "/dev/null"
+        os.open(REDIRECT_TO, os.O_RDWR)	# standard input (0)
+        os.dup2(0, 1)			# standard output (1)
+        os.dup2(0, 2)			# standard error (2)
+
     # write the pid file
     pid_file = "/var/run/firewalld.pid"
     f = open(pid_file, "w")
-- 
1.7.10.4

